<?php

namespace Gateway;
use Connection;
use PDO;


class CharacterGateway extends Gateway
{

    /**
     * Creates a new character record in the database.
     *
     * This method inserts a new character into the `caracter` table using the provided character 
     * ID, name, and image path.
     *
     * @param int $id_character The unique ID for the character.
     * @param string $name The name of the character.
     * @param string $img_char The path to the character's image.
     * 
     * @return bool Returns `true` if the character was successfully created, 
     *              or `false` if the insertion failed (e.g., due to a database error).
     */
    public function create(int $id_character, string $name, string $img_char) : bool 
    {
        // SQL query to insert a new character into the "caracter" table
        $query = "INSERT INTO caracter
            VALUES(:id_caracter, :caracter, :id_img)";

        // Execute the query with the provided parameters
        return $this->co->executeQuery($query, [
            'id_caracter' => array($id_character, PDO::PARAM_INT),
            'caracter' => array($name, PDO::PARAM_STR),
            'id_img' => array($img_char, PDO::PARAM_STR)
        ]);
    }


    /**
     * Finds a character by its ID in the database.
     *
     * This method retrieves a character from the `caracter` table by its unique character ID.
     *
     * @param int $id The unique ID of the character to retrieve.
     * 
     * @return array Returns an array containing the character data if found, or an empty array if no matching character is found.
     */
    public function findById(int $id) : array
    {
        // SQL query to select a character by its unique ID
        $query = "SELECT * FROM caracter WHERE id_caracter = :id_c";

        // Execute the query with the provided character ID
        $this->co->executeQuery($query, ["id_c" => array($id, PDO::PARAM_INT)]);

        // Return the results (array of matching character data)
        return $this->co->getResults();
    }


    /**
     * Finds a character by its name in the database.
     *
     * This method retrieves a character from the `caracter` table by its name.
     *
     * @param string $name The name of the character to retrieve.
     * 
     * @return array Returns an array containing the character data if found, or an empty array if no matching character is found.
     */
    public function findByName(string $name) : array
    {
        // SQL query to select a character by its name
        $query = "SELECT * FROM caracter WHERE caracter = :n";

        // Execute the query with the provided character name
        $this->co->executeQuery($query, ["n" => array($name, PDO::PARAM_STR)]);

        // Return the results (array of matching character data)
        return $this->co->getResults();
    }


    /**
     * Retrieves all characters from the database.
     *
     * This method fetches all rows from the `caracter` table, ordering them by the character name (`caracter`) in ascending order.
     *
     * @return array Returns an array of all characters. Each character is represented as an associative array with column names as keys.
     */
    public function findAll() : array
    {
        // SQL query to retrieve all characters from the 'caracter' table, ordered by character name in ascending order
        $query = "SELECT * FROM caracter ORDER BY caracter ASC";

        // Execute the query
        $this->co->executeQuery($query);

        // Return the results, which is an array of all character records
        return $this->co->getResults();
    }


    /**
     * Deletes a character from the database by its ID.
     *
     * This method removes a character from the `caracter` table based on the provided character ID (`id_caracter`).
     * It executes a DELETE SQL query to remove the record from the database.
     *
     * @param int $id The ID of the character to be deleted.
     * 
     * @return bool Returns `true` if the deletion was successful, `false` otherwise.
     */
    public function delete(int $id) : bool
    {
        // SQL query to delete a character from the 'caracter' table by character ID
        $query = "DELETE FROM caracter WHERE id_caracter = :id_c";

        // Execute the query using the provided character ID, binding the parameter to an integer
        return $this -> co -> executeQuery($query, ["id_c" => array($id, PDO::PARAM_INT)]);
    }


    /**
     * Updates the details of a character in the database.
     *
     * This method updates a character's name and associated image in the `caracter` table
     * based on the provided character ID (`id_caracter`).
     *
     * @param int $id_char The ID of the character to be updated.
     * @param string $name The new name for the character.
     * @param string $img_char The new image path for the character.
     * 
     * @return bool Returns `true` if the update was successful, `false` otherwise.
     */
    public function update(int $id_char, string $name, string $img_char) : bool
    {
        // SQL query to update the character's name and image based on character ID
        $query = "UPDATE caracter
            SET caracter = :n, id_img = :i 
            WHERE id_caracter = :id_c";

        // Execute the query, binding the parameters for character ID, name, and image path
        return $this -> co -> executeQuery($query, [
            "id_c" => array($id_char, PDO::PARAM_INT),
            "caracter" => array($name, PDO::PARAM_STR),
            "i" => array($img_char, PDO::PARAM_STR)
        ]);
    }
}