<?php
namespace Gateway;
use PDO;

class CommentaryGateway  extends Gateway{

    /**
     * Finds the first available ID for a new comment in the Commentary table.
     *
     * This method retrieves all existing comment IDs from the Commentary table, determines the first unused ID,
     * and returns it. If there are no comments in the table, it returns `1` as the starting ID.
     *
     * @return int The first available ID for a new comment.
     */
    public function firstIdComment(): int
    {
        // Define the query to fetch all comment IDs from the Commentary table
        $query = "SELECT id_comment FROM Commentary;";

        // Execute the query and fetch the results
        $this->co->executeQuery($query);
        $res = $this->co->getResults();

        // Initialize an array to store existing comment IDs
        $tab = null;

        // Populate the array with IDs from the result set
        foreach ($res as $r) {
            $tab[] = $r["id_comment"];
        }

        // If no comments exist, return 1 as the starting ID
        if ($tab == null) {
            return 1;
        }

        // Initialize the ID counter
        $id = 1;

        // Increment the ID until a gap is found (i.e., an ID that is not in the array)
        while (in_array($id, $tab)) {
            $id = $id + 1;
        }

        // Return the first available ID
        return $id;
    }


    /**
     * Creates a new comment in the database.
     *
     * This method inserts a new comment for a given quote and user into the `Commentary` table. 
     * It automatically generates a unique comment ID by calling the `firstIdComment()` method.
     *
     * @param string $comment The text content of the comment.
     * @param string $idUser The ID of the user who is posting the comment.
     * @param int $idQuote The ID of the quote that the comment is associated with.
     * 
     * @return bool `true` if the comment was successfully created, `false` otherwise.
     */
    public function create(string $comment, string $idUser, int $idQuote): bool
    {
        // Get the first available comment ID using the firstIdComment method
        $id = $this->firstIdComment();

        // Prepare the SQL query to insert a new comment
        $query = "INSERT INTO Commentary(quote, users, datec, comment, id_comment)
                VALUES(:idQuote, :idUser, CURRENT_DATE, :comment, :idComment)";

        // Execute the query with the provided parameters
        return $this->co->executeQuery($query, array(
            "comment" => array($comment, PDO::PARAM_STR),
            "idUser" => array($idUser, PDO::PARAM_STR),
            "idQuote" => array($idQuote, PDO::PARAM_INT),
            "idComment" => array($id, PDO::PARAM_INT)
        ));
    }


    /**
     * Retrieves a commentary by its unique ID.
     *
     * This method queries the `Commentary` table to find a comment by its `id_comment`.
     * It returns an array containing the details of the comment if found.
     *
     * @param int $id The unique ID of the comment to retrieve.
     * 
     * @return array An array containing the commentary details (or an empty array if no comment is found).
     */
    public function findById(int $id): array
    {
        // SQL query to find the commentary by its unique ID
        $query = "SELECT * 
                  FROM Commentary 
                  WHERE id_comment = :id_comment";

        // Execute the query with the provided comment ID as a parameter
        $this->co->executeQuery($query, array("id_comment" => $id));

        // Return the results from the query execution
        return $this->co->getResults();
    }


    /**
     * Retrieves all comments associated with a specific quote.
     *
     * This method queries the `Commentary`, `Users`, and `Image` tables to find all comments related
     * to a specific quote, and returns the details of each comment, including the commenter's username,
     * the commenter's profile image path, and the comment itself, ordered by date in descending order.
     *
     * @param int $id The ID of the quote for which comments are being retrieved.
     * 
     * @return array An array containing the details of all comments associated with the specified quote.
     *               Each entry includes the comment ID, comment date, content, username, and profile image path.
     */
    public function findByQuote(int $id): array
    {
        // SQL query to retrieve comments for a specific quote, joining with the Users and Image tables
        $query = "SELECT c.id_comment, c.dateC, c.comment, u.username, i.imgPath 
            FROM Commentary c 
            JOIN Users u ON u.id_user = c.users 
            JOIN Image i ON i.id_img = u.img 
            WHERE quote = :idQuote 
            ORDER BY c.datec DESC";

        // Execute the query with the provided quote ID as a parameter
        $this->co->executeQuery($query, array("idQuote" => array($id, PDO::PARAM_STR)));

        // Return the results from the query execution
        return $this->co->getResults();
    }



    /**
     * Retrieves all comments from the database.
     *
     * This method executes a query to fetch all records from the `Commentary` table, 
     * and returns the details of each comment stored in the database.
     *
     * @return array An array containing all the comments in the `Commentary` table.
     *               Each entry includes details of the comment, such as the comment's ID, 
     *               creation date, content, user who made the comment, and the quote associated with it.
     */
    public function findAll() : array
    {
        // SQL query to retrieve all comments from the `Commentary` table
        $query = "SELECT * FROM Commentary";

        // Execute the query
        $this->co->executeQuery($query);

        // Return the results from the query execution
        return $this->co->getResults();
    }


    /**
     * Deletes a comment from the database.
     *
     * This method executes a query to delete a specific comment from the `Commentary` table
     * based on the provided comment ID. It will remove the comment with the given ID if it exists.
     *
     * @param int $id The ID of the comment to be deleted.
     * 
     * @return bool Returns `true` if the comment was successfully deleted, 
     *              or `false` if the deletion failed (e.g., if the comment ID was not found).
     */
    public function delete(int $id) : bool
    {
        // SQL query to delete a comment with the specified ID
        $query = "DELETE FROM Commentary 
                         WHERE id_comment = :id_comment";

        // Execute the query and return the result
        return $this->co->executeQuery($query, array("id_comment" => $id));
    }


    /**
     * Updates a comment in the database.
     *
     * This method executes a query to update the `comment` field of a specific comment in the 
     * `Commentary` table based on the provided `commentaryEntity` object. It will update the 
     * comment with the given ID if it exists.
     *
     * @param commentaryEntity $c The commentary entity containing the new comment content and comment ID.
     * 
     * @return bool Returns `true` if the comment was successfully updated, 
     *              or `false` if the update failed (e.g., if the comment ID was not found).
     */
    public function update(commentaryEntity $c) : bool 
    {
        // SQL query to update the comment based on its ID
        $query = "UPDATE Commentary 
                  SET comment = :comment 
                  WHERE id_comment = :id_comment";

        // Execute the query with the parameters from the commentary entity object
        return $this->co->executeQuery($query, array(
            "comment" => array($c->getComment(), PDO::PARAM_STR),
            "id_comment" => array($c->getIdComment(), PDO::PARAM_INT)
        ));
    }
}
?>
