<?php

namespace Gateway;
use PDO;

class FavoriteGateway extends Gateway
{

    /**
     * Adds a new favorite entry to the `Favorite` table.
     *
     * This method inserts a record into the `Favorite` table, linking a user (`idUser`) with
     * a specific quote (`idQuote`). It assumes that the `Favorite` table has two columns:
     * - `user` (foreign key referencing the user's ID)
     * - `quote` (foreign key referencing the quote's ID)
     *
     * @param int $idUser The ID of the user who is marking the quote as a favorite.
     *                    Must correspond to an existing user in the database.
     * @param int $idQuote The ID of the quote being marked as a favorite.
     *                     Must correspond to an existing quote in the database.
     *
     * @return bool Returns `true` if the query was successfully executed, indicating the favorite
     *              was added. Returns `false` if the query execution failed.
     */
    public function createFavoriteGateway(int $idUser, int $idQuote) : bool
    {
        // Define the SQL query to insert a new favorite entry
        $query = "INSERT INTO Favorite
            VALUES (:user, :quote)";

        // Execute the query using the provided database connection and return the result
        return $this->co->executeQuery($query, [
            'user' => array($idUser, PDO::PARAM_INT),
            'quote' => array($idQuote, PDO::PARAM_INT)
        ]);
    }



    /**
     * Retrieves a specific favorite entry from the `Favorite` table.
     *
     * This method fetches a record from the `Favorite` table based on the provided user ID (`idUser`)
     * and quote ID (`idQuote`). If the favorite exists, it returns the corresponding data as an array.
     *
     * @param int $idUser The ID of the user who favorited the quote. This must reference a valid user in the database.
     * @param int $idQuote The ID of the quote that is favorited. This must reference a valid quote in the database.
     *
     * @return array Returns an associative array containing the favorite record if it exists.
     *               If no matching record is found, an empty array is returned.
     */
    public function findFavoriteById(int $idUser, int $idQuote) : array
    {
        // Define the SQL query to fetch the favorite record
        $query = "SELECT * FROM Favorite
            WHERE user_f = :user AND quote_f = :quote";

        // Execute the query using the provided database connection
        $this->co->executeQuery($query, [
            'user' => array($idUser, PDO::PARAM_INT),
            'quote' => array($idQuote, PDO::PARAM_INT)
        ]);

        // Return the query results as an array
        return $this->co->getResults();
    }


    /**
     * Retrieves all favorites for a specific user from the `Favorite` table.
     *
     * This method fetches all records from the `Favorite` table where the `user_f` column
     * matches the provided user ID. It allows retrieving all quotes favorited by a specific user.
     *
     * @param int $idUser The ID of the user whose favorites are being retrieved. This must reference a valid user in the database.
     *
     * @return array Returns an array of associative arrays, where each entry represents a favorite record.
     *               If the user has no favorites, an empty array is returned.
     */
    public function findFavoriteByUser(int $idUser) : array
    {
        // Define the SQL query to fetch all favorites for the user
        $query = "SELECT * FROM Favorite
            WHERE user_f = :user";

        // Execute the query using the provided database connection
        $this->co->executeQuery($query, ['user' => array($idUser, PDO::PARAM_INT)]);

        // Return the query results as an array
        return $this->co->getResults();
    }



    /**
     * Retrieves all records from the `Favorite` table.
     *
     * This method fetches all the favorite records in the database, returning an array of
     * all user-quote associations stored in the `Favorite` table.
     * 
     * Use this method to get a global view of all user favorites, which can be useful for 
     * administrative tasks or debugging.
     *
     * @return array Returns an array of associative arrays where each entry represents a favorite record.
     *               If there are no records in the table, an empty array is returned.
     */
    public function findAllFavorite() : array
    {
        // Define the SQL query to fetch all favorite records
        $query = "SELECT * FROM Favorite";

        // Execute the query using the provided database connection
        $this->co->executeQuery($query);

        // Return the results of the query as an array
        return $this->co->getResults();
    }


    /**
     * Deletes a specific favorite record from the `Favorite` table.
     *
     * This method removes the association between a user and a quote in the `Favorite` table
     * by matching the provided user ID and quote ID. It is typically used to allow users
     * to unfavorite quotes or to clean up favorites.
     *
     * @param int $idUser The ID of the user whose favorite record is to be deleted. This must reference a valid user in the database.
     * @param int $idQuote The ID of the quote to be removed from the user's favorites. This must reference a valid quote in the database.
     *
     * @return bool Returns `true` if the deletion query was executed successfully, otherwise `false`.
     */
    public function deleteFavoriteGateway(int $idUser, int $idQuote) : bool
    {
        // Define the SQL query to delete a favorite record matching user ID and quote ID
        $query = "DELETE FROM Favorite
            WHERE user_f = :user AND quote_f = :quote";

        // Execute the query with the provided parameters
        return $this->co->executeQuery($query, [
            'user' => array($idUser, PDO::PARAM_INT),
            'quote' => array($idQuote, PDO::PARAM_INT)
        ]);
    }
}