<?php

namespace Gateway;
use PDO;

class ImageGateway extends Gateway
{

    /**
     * Inserts a new image entry into the `Image` table in the database.
     *
     * This method executes an SQL query to insert a new record into the `Image` table with the specified image ID 
     * and image path. It uses the provided database connection to perform the insertion operation.
     * 
     * The image data inserted includes:
     * - The image ID (`idImg`).
     * - The path to the image file (`imgPath`).
     * 
     * **Note**: This method assumes that the `Image` table exists with at least the `id_img` and `img_path` columns.
     *
     * @param int $idImg The unique identifier for the image.
     * @param string $imgPath The file path to the image (including the filename).
     * 
     * @return bool Returns `true` if the query was successfully executed, `false` if there was an error.
     */
    public function createImgGateway(int $idImg, string $imgPath) : bool
    {
        // Define the SQL query to insert a new image record
        $query = "INSERT INTO Image
            VALUES (:id_img, :img_path)";

        // Execute the query using the provided database connection and parameterized values
        return $this->co->executeQuery($query, [
            'id_img' => array($idImg, PDO::PARAM_INT),   // Bind the image ID as an integer
            'img_path' => array($imgPath, PDO::PARAM_STR), // Bind the image path as a string
        ]);
    }


    /**
     * Finds an image record from the `Image` table by its ID.
     *
     * This method executes a SQL query to retrieve the image record corresponding to the provided image ID (`$idImg`).
     * The method returns all columns of the matching image from the `Image` table.
     * 
     * **Note**: This method assumes that the `Image` table exists with at least the `id_img` and `img_path` columns.
     *
     * @param int $idImg The unique identifier for the image to retrieve.
     * 
     * @return array Returns an associative array of the image record. The array contains key-value pairs where 
     *               each key is a column name and the value is the corresponding column value from the `Image` table.
     *               If no matching record is found, it returns an empty array.
     */
    public function findImgById(int $idImg) : array
    {
        // Define the SQL query to select an image by its ID
        $query = "SELECT * FROM Image
            WHERE id_img = :id_img";

        // Execute the query using the provided database connection and parameterized values
        $this->co->executeQuery($query, ['id_img' => array($idImg, PDO::PARAM_INT)]);

        // Return the results of the query execution
        return $this->co->getResults();
    }



    /**
     * Retrieves all image records from the `Image` table.
     *
     * This method executes a SQL query to fetch all records from the `Image` table
     * and returns the results as an associative array. Each record corresponds to one image.
     * 
     * **Note**: This method assumes that the `Image` table exists with appropriate columns, 
     * such as `id_img` and `img_path`.
     *
     * @return array Returns an array of associative arrays, where each associative array represents
     *               a single image record from the `Image` table. Each key in the associative array
     *               corresponds to a column name, and the value is the respective column value.
     *               If no records are found, returns an empty array.
     */
    public function findAllImg() : array
    {
        // Define the SQL query to retrieve all image records
        $query = "SELECT * FROM Image";

        // Execute the query using the provided database connection
        $this->co->executeQuery($query);

        // Return the results of the query execution
        return $this->co->getResults();
    }



    /**
     * Deletes an image record from the `Image` table based on the provided ID.
     *
     * This method executes a SQL query to delete a specific image record from the database,
     * identified by its unique `id_img`.
     * 
     * **Note**: This method assumes that the `Image` table exists and the `id_img` column
     * is used as the primary key or a unique identifier for each image.
     *
     * @param int $idImg The ID of the image to be deleted from the `Image` table.
     *                   Must be a valid integer corresponding to an existing image record.
     *
     * @return bool Returns `true` if the query was successfully executed, indicating the image
     *              record was deleted. Returns `false` if the query failed.
     */
    public function deleteImgGateway(int $idImg) : bool
    {
        // Define the SQL query to delete an image record based on its ID
        $query = "DELETE FROM Image
            WHERE id_img = :id_img";

        // Execute the query using the provided database connection and return the result
        return $this->co->executeQuery($query, [
            'id_img' => array($idImg, PDO::PARAM_INT)
        ]);
    }


    /**
     * Updates the path of an image in the `Image` table based on the provided image ID.
     *
     * This method executes a SQL query to update the `imgPath` column for a specific image
     * identified by its unique `id_img`. It assumes the `Image` table exists and contains
     * the columns `id_image` and `imgPath`.
     *
     * @param int $idImg The ID of the image to be updated. Must correspond to an existing
     *                   record in the `Image` table.
     * @param string $imgPath The new path for the image. Must be a valid string representing
     *                        the location of the image file.
     *
     * @return bool Returns `true` if the query was successfully executed, indicating that the
     *              image path was updated. Returns `false` if the query execution failed.
     */
    public function updateImgGateway(int $idImg, string $imgPath) : bool
    {
        // Define the SQL query to update the image path based on the image ID
        $query = "UPDATE Image
            SET imgPath = :img_path
            WHERE id_image = :id_img";

        // Execute the query using the provided database connection and return the result
        return $this->co->executeQuery($query, [
            'id_img' => array($idImg, PDO::PARAM_INT),
            'img_path' => array($imgPath, PDO::PARAM_STR)
        ]);
    }
}