<?php

namespace Gateway;
use Gateway\Connection;
use PDO;


class QuizGateway extends Gateway
{

    /**
     * Creates a new quiz in the database.
     *
     * This method inserts a new record into the `Quiz` table, specifying the quiz ID
     * and the number of questions in the quiz.
     *
     * @param int $id_quiz The unique ID of the quiz.
     * @param int $nb_questions The total number of questions in the quiz.
     *
     * @return bool Returns `true` if the query was successfully executed, or `false` otherwise.
     */
    public function create(int $id_quiz, int $nb_questions) : bool
    {
        // SQL query to insert a new quiz into the Quiz table
        $query = "INSERT INTO Quiz
            VALUES (:id_q, :nb_q)";

        // Execute the query with bound parameters
        return $this -> co -> executeQuery($query, [
            ':id_q' => array($id_quiz, PDO::PARAM_INT),      // Bind the quiz ID
            ':nb_q' => array($nb_questions, PDO::PARAM_INT)  // Bind the number of questions
        ]);
    }


    /**
     * Retrieves a quiz from the database by its ID.
     *
     * This method fetches a single quiz record from the `Quiz` table, identified by its unique ID.
     *
     * @param int $id The unique ID of the quiz to retrieve.
     *
     * @return array Returns an associative array representing the quiz, with keys corresponding
     *               to the table columns. If no quiz is found, returns an empty array.
     */
    public function findQuizById(int $id) : array
    {
        // SQL query to fetch a quiz by its ID
        $query = "SELECT * 
                  FROM Quiz 
                  WHERE id_quiz = :id_q";

        // Execute the query with the bound ID parameter
        $this -> co -> executeQuery($query, [
            ':id_q' => array($id, PDO::PARAM_INT) // Bind the quiz ID
        ]);

        // Return the result as an array
        return $this -> co -> getResults();
    }


    /**
     * Deletes a quiz from the database by its ID.
     *
     * This method removes a specific quiz from the `Quiz` table, identified by its unique ID.
     *
     * @param int $id The unique ID of the quiz to be deleted.
     *
     * @return bool Returns `true` if the query was successfully executed, or `false` otherwise.
     */
    public function delete(int $id) : bool
    {
        // SQL query to delete a quiz by its ID
        $query = "DELETE FROM Quiz 
                  WHERE id_quiz = :id_q";

        // Execute the query with the provided ID parameter
        return $this -> co -> executeQuery($query, [
            ':id_q' => array($id, PDO::PARAM_INT) // Bind the quiz ID
        ]);
    }


    /**
     * Retrieves all quizzes from the database.
     *
     * This method fetches all records from the `Quiz` table, returning an array of results.
     *
     * @return array Returns an array containing all quizzes. Each quiz is represented as an associative array with its columns as keys.
     */
    public function findAll() : array
    {
        // SQL query to select all records from the Quiz table
        $query = "SELECT * FROM Quiz";

        // Execute the query to fetch all quiz records
        $this -> co -> executeQuery($query);

        // Return the results retrieved from the database
        return $this -> co -> getResults();
    }


}