<?php
namespace Gateway;
use Enum\TypeSourceEnum;
use Entity\SourceEntity;
use PDO;

class SourceGateway extends Gateway{

    /**
     * Inserts a new source into the Source table.
     *
     * This method inserts a new source into the `Source` table, setting attributes like the title and date of the source.
     * It takes a `sourceEntity` object as an argument, which contains the data required for the insertion. 
     * The method assumes that the source entity is valid and contains the appropriate values for the title and date.
     *
     * @param sourceEntity $s The source entity object containing the source's title and date.
     *
     * @return bool Returns true if the query executed successfully, false otherwise.
     */
    public function create(sourceEntity $s) : bool
    {
        // SQL query to insert a new source into the database
        $query = "INSERT INTO Source
            VALUES( :title, :date)";

        // Execute the query with the parameters extracted from the source entity object
        return $this->co->executeQuery($query, [
            'title' => array($s->getTitle(), PDO::PARAM_STR),  // The title of the source (e.g., book title, movie title)
            'date' => array($s->getDate(), PDO::PARAM_STR),    // The date of the source (e.g., publication date, release date)
            // 'type' => array($s->getType(), PDO::PARAM_STR)  // Optional: Add 'type' if needed (commented out for now)
        ]);
    }

    /**
     * Retrieves a source by its ID from the Source table.
     *
     * This method fetches the details of a source from the `Source` table by its `id_source`.
     * It executes a query to retrieve the source record that matches the provided ID.
     * If a source is found with the given ID, the details of the source are returned as an associative array.
     *
     * @param int $id The ID of the source to be retrieved.
     *
     * @return array An associative array containing the details of the source, or an empty array if no source is found.
     */
    public function findById(int $id) : array
    {
        // SQL query to select a source by its ID
        $query = "SELECT * 
                  FROM Source 
                  WHERE id_source = :id";

        // Execute the query with the provided ID
        $this->co->executeQuery($query, array("id" => array($id, PDO::PARAM_INT)));

        // Return the result of the query
        return $res = $this->co->getResults();
    }


    /**
     * Retrieves a source by its title from the Source table.
     *
     * This method fetches the details of a source from the `Source` table by its `title`.
     * It executes a query to retrieve the source record that matches the provided title.
     * If a source is found with the given title, the details of the source are returned as an associative array.
     *
     * @param string $t The title of the source to be retrieved.
     *
     * @return array An associative array containing the details of the source, or an empty array if no source is found.
     */
    public function findByTitle(string $t) : array
    {
        // SQL query to select a source by its title
        $query = "SELECT * 
                  FROM Source 
                  WHERE title = :t";

        // Execute the query with the provided title
        $this->co->executeQuery($query, ["t" => array($t, PDO::PARAM_STR)]);

        // Return the result of the query
        return $res = $this->co->getResults();
    }


    /**
     * Retrieves all sources from the Source table.
     *
     * This method fetches all records from the `Source` table, sorted by the `title` column in ascending order.
     * It executes a query to retrieve all source records and returns them as an associative array.
     *
     * @return array An array of associative arrays, each containing the details of a source.
     */
    public function findAll() : array
    {
        // SQL query to select all sources from the Source table, ordered by title in ascending order
        $query = "SELECT * 
                  FROM Source 
                  ORDER BY title ASC";

        // Execute the query to fetch the results
        $this->co->executeQuery($query);

        // Return the result of the query
        return $this->co->getResults();
    }


    /**
     * Deletes a source record from the Source table by its ID.
     *
     * This method deletes the record from the `Source` table where the `id_source` matches the provided ID.
     * It executes the DELETE SQL query to remove the source from the database.
     *
     * @param int $id The ID of the source to delete.
     * @return bool True if the delete was successful, false otherwise.
     */
    public function delete(int $id) : bool
    {
        // SQL query to delete the source record with the given id_source
        $query = "DELETE FROM Source 
                  WHERE id_source = :id_s";

        // Execute the query with the provided source ID
        return $this->co->executeQuery($query, ["id_s" => array($id, PDO::PARAM_INT)]);
    }


    /**
     * Updates an existing source record in the Source table.
     *
     * This method updates the `title` and `date` columns of the `Source` table for a given `id_source`.
     * It executes the `UPDATE` SQL query to modify the record with the specified ID.
     *
     * @param sourceEntity $s The source entity containing the updated data (ID, title, and date).
     * @return bool True if the update was successful, false otherwise.
     */
    public function update(sourceEntity $s) : bool
    {
        // SQL query to update the title and date of a source record with the given id_source
        $query = "UPDATE Source 
            SET title = :t, date = :d 
            WHERE id_source = :id_s";

        // Execute the query with the provided source entity data
        return $this->co->executeQuery($query, [
            "id_s" => array($s->getIdSource(), PDO::PARAM_INT),
            "t" => array($s->getTitle(), PDO::PARAM_STR),
            "d" => array($s->getDate(), PDO::PARAM_STR)
        ]);
    }
}
