<?php

    namespace Model;
    use Entity\Quote;
    use Gateway\QuoteGateway;
    use Gateway\Gateway;

    class QuoteModel extends Model
    {

        /**
         * Searches for a quote by its unique identifier.
         *
         * @param int $id The unique identifier of the quote to search for.
         * 
         * @return Quote Returns a Quote object. If no quote is found, returns a default Quote with an ID of -1 and default values.
         */
        public function searchId(int $id): Quote {
            // Fetch the quote data by ID using the gateway's searchId method.
            $res = $this->gateway->searchId($id);

            // If no result is found, return a default Quote with an ID of -1 and placeholder values.
            if (count($res) == 0) {
                return new Quote(-1, "NULL", "NULL", "NULL", "NULL", "NULL", 0, "Default");
            } else {
                // Otherwise, return the Quote object populated with data from the search result.
                return new Quote(
                    $res[0]["id_quote"], // Quote ID
                    $res[0]["content"],   // Quote content
                    $res[0]["caracter"],  // Character associated with the quote
                    $res[0]["imgpath"],   // Image path (associated with the quote or character)
                    $res[0]["title"],     // Title of the quote
                    $res[0]["dates"],     // Date the quote was created
                    $res[0]["likes"],     // Number of likes for the quote
                    $res[0]["langue"]     // Language of the quote
                );
            }
        }

        /**
         * Retrieves suggestions for quotes based on the specified page number and language.
         *
         * @param int $numpage The page number for pagination.
         * @param string $language The language to filter the suggestions by.
         * 
         * @return Quote[] Returns an array of Quote objects representing the suggested quotes.
         */
        public function getSuggest(int $numpage, string $language): array {
            // Fetch suggestions from the gateway's getSuggestions method.
            $res = $this->gateway->getSuggestions($numpage, $language);

            // Initialize an empty array to store the resulting Quote objects.
            $tabQ = [];

            // Loop through the result set and create Quote objects.
            foreach ($res as $q) {
                // Truncate the content if it's longer than 153 characters, appending '...' if necessary.
                $q["content"] = (strlen($q["content"]) > 153) ? substr($q["content"], 0, 150) . '...' : $q["content"];

                // Create a new Quote object and add it to the tabQ array.
                $tabQ[] = new Quote(
                    $q["id_quote"],   // Quote ID
                    $q["content"],     // Truncated content of the quote
                    $q["caracter"],    // Character associated with the quote
                    $q["imgpath"],     // Image path related to the quote
                    $q["title"],       // Title of the quote
                    $q["dates"],       // Date the quote was created
                    $q["likes"],       // Number of likes the quote has received
                    $q["langue"]       // Language of the quote
                );
            }

            // Return the array of Quote objects.
            return $tabQ;
        }

        /**
         * Retrieves the quote of the day for a specified language.
         *
         * @param string $language The language to filter the quote of the day by.
         * 
         * @return Quote Returns the Quote of the Day. If no quote is found, returns a default Quote with an ID of -1 and default values.
         */
        public function getQuoteOfTheDay(string $language): Quote {
            // Fetch the quote of the day from the gateway's getQuoteOfTheDay method.
            $res = $this->gateway->getQuoteOfTheDay($language);

            // If no result is found, return a default Quote with an ID of -1 and placeholder values.
            if (count($res) == 0) {
                return new Quote(-1, "NULL", "NULL", "NULL", "NULL", "NULL", 0, "Default");
            } else {
                // Truncate the content if it's longer than 153 characters, appending '...' if necessary.
                $res["content"] = (strlen($res["content"]) > 153) ? substr($res["content"], 0, 150) . '...' : $res["content"];

                // Return the Quote object populated with data from the quote of the day.
                return new Quote(
                    $res["id_quote"],   // Quote ID
                    $res["content"],     // Truncated content of the quote
                    $res["caracter"],    // Character associated with the quote
                    $res["imgpath"],     // Image path associated with the quote
                    $res["title"],       // Title of the quote
                    $res["dates"],       // Date the quote was created
                    $res["likes"],       // Number of likes the quote has received
                    $res["langue"]       // Language of the quote
                );
            }
        }

        /**
         * Retrieves all favorite quotes for a specified user.
         *
         * @param string $userId The unique identifier of the user.
         * 
         * @return Quote[] Returns an array of Quote objects representing the user's favorite quotes.
         */
        public function getFavorites(string $userId): array {
            // Fetch the user's favorite quotes from the gateway's getFavorites method.
            $res = $this->gateway->getFavorites($userId);

            // Initialize an empty array to store the resulting Quote objects.
            $tabQ = [];

            // Loop through the result set and create Quote objects.
            foreach ($res as $q) {
                // Truncate the content if it's longer than 153 characters, appending '...' if necessary.
                $q["content"] = (strlen($q["content"]) > 153) ? substr($q["content"], 0, 150) . '...' : $q["content"];

                // Create a new Quote object and add it to the tabQ array.
                $tabQ[] = new Quote(
                    $q["id_quote"],   // Quote ID
                    $q["content"],     // Truncated content of the quote
                    $q["caracter"],    // Character associated with the quote
                    $q["imgpath"],     // Image path related to the quote
                    $q["title"],       // Title of the quote
                    $q["dates"],       // Date the quote was created
                    $q["likes"],       // Number of likes the quote has received
                    $q["langue"]       // Language of the quote
                );
            }

            // Return the array of favorite Quote objects.
            return $tabQ;
        }

        /**
         * Inserts a new quote for a specific user.
         *
         * @param string $content The content of the quote.
         * @param string $img_path The image path associated with the quote.
         * @param string $langage The language of the quote.
         * @param int $user The ID of the user associated with the quote.
         * @param int $source The source of the quote (e.g., the origin of the quote or where it was sourced from).
         * @param int $character The character associated with the quote.
         * 
         * @return bool Returns true if the quote was successfully inserted, otherwise false.
         */
        public function insert4User(string $content, string $img_path, string $langage, int $user, int $source, int $character): bool {
            // Call the gateway's insert4User method to insert the new quote into the database.
            return $this->gateway->insert4User($content, $img_path, $langage, $user, $source, $character);
        }

    }

?>
