<?php

namespace Model;
use Entity\SourceEntity;
use Enum\TypeSourceEnum;


class SourceModel extends Model
{

    /**
     * Creates a new source with the provided title, date, and type.
     *
     * @param string $title The title of the source.
     * @param string $date The date associated with the source.
     * @param TypeSourceEnum $type The type of the source (e.g., Movie, Book, etc.).
     * 
     * @return bool Returns true if the source was successfully created, false otherwise.
     */
    public function createSource(string $title, string $date, TypeSourceEnum $type) : bool
    {
        // Create a new SourceEntity with the given title, date, and type.
        // Note: The type is hardcoded as "Movie" in this implementation.
        $q = new SourceEntity(-1, $title, $date, TypeSourceEnum::Movie);

        // Use the gateway to insert the new source and return the result.
        return $this->gateway->create($q);
    }

    /**
     * Retrieves a source by its unique identifier.
     *
     * @param int $id_source The unique identifier of the source.
     * 
     * @return SourceEntity|null Returns a SourceEntity object if found, or null if no source is found with the given ID.
     */
    public function getSourceById(int $id_source) : ?SourceEntity
    {
        // Fetch the source by ID using the gateway's findById method.
        $res = $this->gateway->findById($id_source);

        // If a source is found, return a SourceEntity object.
        if ($res) {
            return new SourceEntity(
                $res[0]["id_source"],  // Source ID
                $res[0]["title"],      // Title of the source
                $res[0]["dates"],      // Date associated with the source
                TypeSourceEnum::Movie  // Hardcoded to "Movie" type; you may consider dynamically mapping this
            );
        }
        // Return null if no source is found.
        return null;
    }

    /**
     * Retrieves a source by its title.
     *
     * @param string $title The title of the source to search for.
     * 
     * @return SourceEntity|null Returns a SourceEntity object if found, or null if no source is found with the given title.
     */
    public function getSourceByTitle(string $title) : ?SourceEntity
    {
        // Fetch the source by title using the gateway's findByTitle method.
        $res = $this->gateway->findByTitle($title);

        // If a source is found, return a SourceEntity object.
        if ($res) {
            return new SourceEntity(
                $res[0]["id_source"],  // Source ID
                $res[0]["title"],      // Title of the source
                $res[0]["dates"],      // Date associated with the source
                TypeSourceEnum::Movie  // Hardcoded to "Movie" type; dynamic mapping could be considered
            );
        }
        // Return null if no source is found.
        return null;
    }

    /**
     * Retrieves all sources from the database.
     *
     * @return SourceEntity[] Returns an array of SourceEntity objects representing all the sources.
     */
    public function getAllSources() : array
    {
        // Fetch all sources from the gateway's findAll method.
        $res = $this->gateway->findAll();

        // Initialize an empty array to store the SourceEntity objects.
        $src = [];

        // Loop through the result and create a SourceEntity object for each source.
        foreach ($res as $sources) {
            $src[] = new SourceEntity(
                $sources["id_source"],  // Source ID
                $sources["title"],      // Title of the source
                $sources["dates"],      // Date associated with the source
                TypeSourceEnum::Movie   // Hardcoded to "Movie" type; you may consider dynamically mapping this
            );
        }

        // Return the array of SourceEntity objects.
        return $src;
    }

    /**
     * Checks if a source with the given name and type exists.
     *
     * @param string $name The name of the source to check.
     * @param string $type The type of the source to check (e.g., Movie, Book, etc.).
     * 
     * @return bool Returns true if the source exists, false otherwise.
     */
    public function existSource(string $name, string $type) : bool
    {
        // Retrieve the source by title using the getSourceByTitle method.
        $q = $this->getSourceByTitle($name);

        // Check if the source exists and return the result.
        return isset($q[0]);
    }

}

?>